clear all;
% Sparse system identification with different algorithms
% Please acknowledge the paper
% R. C. de Lamare and R. Sampaio-Neto, "Sparsity-Aware 
% Adaptive Algorithms Based on Alternating Optimization 
% and Shrinkage", IEEE Signal Processing Letters, vol. 21, 
% no. 2, February 2014.
% 
I = 10; %			-	number of repetitions
K = 300; %			-	iterations
N = 32; %           -   length of the system
S = 2;  %           -   number of nonzero coefficients
sigmax = 1; %       -	standard deviation of the input signal
c = complex(randn(S,1),randn(S,1)); c=c/sqrt(c'*c); % generation of S complex coefficients
ind = round(1 + (N-1).*rand(S,1)); % -	sample index
Wo  = zeros(N,1); 
Wo(ind)  = c; Wo = Wo/sqrt(Wo'*Wo); %      -	plant/system to be identified
sigman = sqrt(0.001); %    -	standard deviation of the noise
mu = 0.25;  %		-	step size
mu_nlms = 0.25;
mu_or = 0.1;
mu_m = 0.1;
mu_a = 0.1;
ep = 10.0; %
reg = 0.01;
gamma = 0.0001; %
gamma_a = 0.0001;
% MSE vectors 
MSE_nlms = zeros(K,1);
MSE_rza = zeros(K,1);
MSE_alt = zeros(K,1);
MSE_or = zeros(K,1);
% Parameters of the oracle algorithm
p = zeros(N,1);
p(ind) = 1;
P = diag(p);
% MSD vectors
MSD_nlms = zeros(K,1);	
MSD_rza = zeros(K,1);
MSD_or = zeros(K,1);
MSD_alt = zeros(K,1);
% Minimum MSE vector
MSEmin=zeros(K,1);
rho = 0.01;
Q = 8*N;
for i=1:I,		
   t = randn(N,1)*sigmax;  
   y = zeros(N,1); y(ind) = t(ind);  	
   w_nlms = zeros(N,1);
   w_rza = zeros(N,1);
   w_alt = zeros(N,1);
   a_alt = 1*ones(N,1);
   w_or = zeros(N,1);
   % input
   x = randn(K,1)*sigmax;		
   % noise
   n = complex(randn(K,1),randn(K,1))*sigman; 		 
   for k = 1:K,		
      y = [x(k) 
          y(1:N-1) ];
      % desired signal 
      d = Wo'*y;	
      % output estimates 
      z_nlms = w_nlms'*y;	
      z_rza = w_rza'*y;
      z_alt = w_alt'*diag(a_alt)*y;
      z_or = w_or'*P*y;
      % error signal
      e_nlms = d+n(k)- z_nlms;	  
      e_rza = d+n(k)- z_rza;	
      e_alt = d + n(k) - z_alt;
      e_or = d + n(k) - z_or;
      % new/updated filters
      w_nlms = w_nlms + mu_nlms/(reg + y'*y)*conj(e_nlms)*y; %/(reg + y'*y)
      w_rza = w_rza + mu/(reg + y'*y)*conj(e_rza)*y - gamma.*(sign(real(w_rza)) + j*sign(imag(w_rza)) )./(1+ep.*abs(w_rza));	%/(reg + y'*y)
      a_alt = a_alt + mu_a/(reg + y'*diag(w_alt')*diag(w_alt)*y)*e_alt*diag(w_alt)*conj(y);% /(reg + y'*y)   - gamma_a.*(sign(real(a_alt))+j*sign(imag(a_alt)))./(1+ep.*abs(a_alt));
      e_alt = d + n(k) - w_alt'*diag(a_alt)*y;
      w_alt = w_alt  + mu/(reg + y'*diag(a_alt)*diag(a_alt')*y)*conj(e_alt)*diag(a_alt)*y - gamma.*(sign(real(w_alt)) + j*sign(imag(w_alt)) )./(1+ep.*abs(w_alt)); % /(reg + y'*y)
      w_or = w_or + mu_or/(reg + y'*P*y)*conj(e_or)*P*y; %
      if k > Q
          w_alt(abs(w_alt) < rho) = 0; %a_alt(abs(w_alt) < rho) =0;
          %w_rza(abs(w_rza) < rho) = 0;    
          %w_nlms(abs(w_nlms) < rho) = 0;
      end
      % accummulation of MSE
      MSE_nlms(k) = MSE_nlms(k)+abs(e_nlms)^2;	
      MSE_rza(k) = MSE_rza(k)+abs(e_rza)^2;	
      MSE_alt(k) = MSE_alt(k)+abs(e_alt)^2;	
      MSE_or(k) = MSE_or(k)+abs(e_or)^2;
      % accummulation of MSE
      MSEmin(k) = MSEmin(k)+abs(n(k))^2;
      % accummulation of MSD
      MSD_nlms(k) = MSD_nlms(k)+(w_nlms-Wo)'*(w_nlms-Wo); %/(Wo'*Wo);
      MSD_rza(k) = MSD_rza(k)+(w_rza-Wo)'*(w_rza-Wo);
      MSD_alt(k) = MSD_alt(k)+(diag(a_alt')*w_alt-Wo)'*(diag(a_alt')*w_alt-Wo);
      MSD_or(k) = MSD_or(k)+(w_or-Wo)'*(w_or-Wo);
   end
end
% sample index
ind=1:(K);
MSE_nlms = MSE_nlms/I;
MSE_rza = MSE_rza/I;
MSE_alt = MSE_alt/I;
MSE_or = MSE_or/I;
MSD_nlms = MSD_nlms/I;
MSD_rza = MSD_rza/I;
MSD_alt = MSD_alt/I;
MSD_or = MSD_or/I;
MSEmin = MSEmin/I;
% Misadjustment computation 
M=MSE_nlms./MSEmin-1; 
scal = [1 50:50:K];
ind = ind(scal);
MSE_nlms = MSE_nlms(scal); MSE_rza = MSE_rza(scal); MSE_alt = MSE_alt(scal); MSE_or = MSE_or(scal);
plot(ind,10*log10(MSE_nlms),'b-o',ind,10*log10(MSE_rza),'r-s',ind,10*log10(MSE_alt),'m-*',ind,10*log10(MSE_or),'g-<');
%semilogy(ind,(MSE_nlms),'b',ind,(MSE_rza),'r',ind,(MSE_or),'g');
xlabel('iterations');
ylabel('MSE');
legend('NLMS','RZA-NLMS','Alt-NLMS','Oracle-NLMS');
axis([0 K (10*log10(sigman^2)-5) 5]);
figure;
MSD_nlms = MSD_nlms(scal); MSD_rza = MSD_rza(scal); MSD_alt = MSD_alt(scal); MSD_or = MSD_or(scal);
plot(ind,10*log10(MSD_nlms),'b-o',ind,10*log10(MSD_rza),'r-s',ind,10*log10(MSD_alt),'m-*',ind,10*log10(MSD_or),'g-<');
xlabel('iterations');
ylabel('MSD');
legend('NLMS','RZA-NLMS','Alt-NLMS','Oracle-NLMS');
%axis([0 K (10*log10(sigman^2)-5) 5]);

% save mse_1-1000_lms MSE_nlms MSE_rza MSE_alt MSE_m MSE_or MSD_nlms
% MSD_rza MSD_alt MSD_or;

% save mse_1000-2000_K4lms MSE_nlms MSE_rza MSE_alt MSE_m MSE_or MSD_nlms
% MSD_rza MSD_alt MSD_or;

%save mse_K2&K4 mse_lms mse_za mse_rza mse_l0 mse_alt_za mse_alt_rza mse_alt_l0 mse_or;

%save mse_analytical MSE_nlms MSE_rza MSE_alt MSE_m MSE_or;
plot(ind,10*log10(MSE_nlms),'b-o',ind,10*log10(MSE_rza),'r-s',ind,10*log10(MSE_alt),'k-+',ind,10*log10(MSE_or),'g-<',ind,10*log10(MSE_nlms_a),'b:',ind,10*log10(MSE_rza_a),'r:',ind,10*log10(MSE_alt_a),'k:');
%semilogy(ind,(MSE_nlms),'b',ind,(MSE_rza),'r',ind,(MSE_or),'g');
xlabel('iterations');
ylabel('MSE');
legend('NLMS','RZA-NLMS','Alt-NLMS','Oracle-NLMS');
axis([0 K (10*log10(sigman^2)-5) 5]);

% ind = [1 50 100:50:2000];
% plot(ind,10*log10(mse_lms),'b-o',ind,10*log10(mse_rza),'r-s',ind,10*log10(mse_l0),'g->',ind,10*log10(mse_alt_rza),'m-*',ind,10*log10(mse_alt_l0),'c-<',ind,10*log10(mse_or),'k');
% xlabel('iterations');
% ylabel('MSE');
% legend('LMS','SA-LMS(RZA)','SA-LMS(L0-Norm)','SA-ALT-LMS(RZA)','SA-ALT-LMS(L0-Norm)','Oracle-LMS');
% axis([0 2000 (10*log10(sigman^2)-5) 5]);

% subplot(3,1,1);
% subplot(1,3,1);
% semilogx(step,(MSE_sim_za),'b-o',step,(MSE_anal_za),'b:+');
% subplot(1,3,2);
% semilogx(step,(MSE_sim_rza),'r-o',step,(MSE_anal_rza),'r:+');
% subplot(1,3,3);
% semilogx(step,(MSE_sim_l0),'g-o',step,(MSE_anal_l0),'g:+');
% legend('SA-ALT-LMS(ZA)-Simulated','SA-ALT-LMS(ZA)-Analytical');
% legend('SA-ALT-LMS(RZA)-Simulated','SA-ALT-LMS(RZA)-Analytical');
% legend('SA-ALT-LMS(l_0-Norm)-Simulated','SA-ALT-LMS(l_0-Norm)-Analytical');